"""
Brewblox-ctl .env commands
"""


import click
import dotenv

from brewblox_ctl import click_helpers, const, utils


@click.group(cls=click_helpers.OrderedGroup)
def cli():
    """Command collector"""


@cli.group(cls=click_helpers.OrderedGroup)
def env():
    """List, get, or set env values.

    Brewblox stores system settings in the brewblox/.env file.
    These settings are used in docker-compose.yml files, and in brewblox-ctl commands.

    You can add your own, and use them in docker-compose.yml with the `${VAR_NAME}` syntax.
    """


@env.command()
@click.argument('value',
                type=click.Choice(['true', 'false'], case_sensitive=False),
                default='true')
def skip_confirm(value):
    """Auto-answer 'yes' when prompted to confirm commands.

    This sets the 'BREWBLOX_SKIP_CONFIRM' variable in .env.
    You can still use the `brewblox-ctl [--dry-run|--verbose] COMMAND` arguments.
    """
    utils.check_config()
    utils.confirm_mode()
    utils.setenv(const.SKIP_CONFIRM_KEY, value.lower())


@env.command(name='list')
def list_env():
    """List all .env variables

    This does not include other variables set in the current shell.
    """
    utils.check_config()
    for k, v in dotenv.dotenv_values('.env').items():
        click.echo('{} = {}'.format(k, v))


@env.command(name='get')
@click.argument('key')
@click.argument('default', default='')
def get_env(key, default):
    """Read a single env variable.

    This includes values from .env, but also other shell values.
    """
    click.echo(utils.getenv(key, default))


@env.command(name='set')
@click.argument('key')
@click.argument('value')
def set_env(key, value):
    """Set a .env variable.

    The value will be added to the .env file. You can set new variables with this.
    """
    utils.check_config()
    utils.confirm_mode()
    utils.setenv(key, value)



"""
Brewblox-ctl snapshot commands
"""

from os import listdir, path
from tempfile import TemporaryDirectory

import click
from brewblox_ctl import click_helpers, utils
from brewblox_ctl.utils import sh


@click.group(cls=click_helpers.OrderedGroup)
def cli():
    """Command collector"""


@cli.group()
def snapshot():
    """Save or load snapshots."""


@snapshot.command()
@click.option('--dir',
              help='Brewblox directory')
@click.option('--file',
              help='Snapshot file')
@click.option('--force',
              is_flag=True,
              help='Remove previous tarfile if it exists')
def save(dir, file, force):
    """Save Brewblox directory to snapshot.

    This can be used to move Brewblox installations between hosts.
    To load the snapshot, use `brewblox-ctl install --snapshot ARCHIVE`
    or `brewblox-ctl snapshot load --file ARCHIVE`

    Block data stored on Spark controllers is not included in the snapshot.
    """
    utils.confirm_mode()

    if utils.is_brewblox_cwd():
        dir = dir or '.'
        file = file or '../brewblox.tar.gz'
    else:
        dir = dir or './brewblox'
        file = file or './brewblox.tar.gz'

    dir = path.abspath(dir)

    if not utils.is_brewblox_dir(dir):
        raise ValueError('`{}` is not a Brewblox directory'.format(dir))

    if utils.path_exists(file):
        if force or utils.confirm('`{}` already exists. '.format(file) +
                                  'Do you want to overwrite it?'):
            sh('rm -f {}'.format(file))
        else:
            return

    basedir = path.basename(dir)
    parent = dir + '/..'
    sh('sudo tar -C {} -czf {} {}'.format(parent, file, basedir))
    click.echo(path.abspath(path.expanduser(file)))


@snapshot.command()
@click.option('--dir',
              help='Brewblox directory')
@click.option('--file',
              help='Snapshot file')
@click.option('--force',
              is_flag=True,
              help='Remove directory if it exists')
def load(dir, file, force):
    """Create Brewblox directory from snapshot.

    This can be used to move Brewblox installations between hosts.
    To create a snapshot, use `brewblox-ctl snapshot save`
    """
    utils.confirm_mode()

    if utils.is_brewblox_cwd():
        dir = dir or '.'
        file = file or '../brewblox.tar.gz'
    else:
        dir = dir or './brewblox'
        file = file or './brewblox.tar.gz'

    dir = path.abspath(dir)

    if utils.path_exists(dir) and not utils.is_empty_dir(dir):
        if not utils.is_brewblox_dir(dir):
            raise FileExistsError('`{}` is not a Brewblox directory.'.format(dir))
        if force or utils.confirm('`{}` already exists. '.format(dir) +
                                  'Do you want to continue and erase its content?'):
            # we'll do the actual rm after unpacking files
            utils.info('Contents of `{}` will be removed'.format(dir))
        else:
            return

    with TemporaryDirectory() as tmpdir:
        utils.info('Extracting snapshot to {} directory...'.format(dir))
        sh('tar -xzf {} -C {}'.format(file, tmpdir))
        content = listdir(tmpdir)
        if utils.ctx_opts().dry_run:
            content = ['brewblox']
        if len(content) != 1:
            raise ValueError('Multiple files found in snapshot: {}'.format(content))
        sh('mkdir -p {}'.format(dir))
        sh('sudo rm -rf {}/*'.format(dir))
        # We need to explicitly include dotfiles in the mv glob
        src = '{}/{}'.format(tmpdir, content[0])
        sh('mv {}/.[!.]* {}/* {}/'.format(src, src, dir))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['brewblox_ctl', 'brewblox_ctl.commands']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.1.1,<8.0.0',
 'configobj>=5.0.6,<6.0.0',
 'docker-compose>=1.26.2,<2.0.0',
 'docker>=4.3.1,<5.0.0',
 'python-dotenv[cli]>=0.13.0,<0.14.0',
 'pyyaml>=5.3.1,<6.0.0',
 'requests>=2.23.0,<3.0.0',
 'zeroconf>=0.28.1,<0.29.0']

entry_points = \
{'console_scripts': ['brewblox-ctl = brewblox_ctl.__main__:main']}

setup_kwargs = {
    'name': 'brewblox-ctl',
    'version': '0.23.1',
    'description': 'Brewblox management tool',
    'long_description': '# BrewBlox CLI management tool\n\nThe primary tool for installing and managing BrewBlox. Uses Click.\n\nInstall-specific commands are defined in [brewblox-ctl-lib](https://github.com/BrewBlox/brewblox-ctl-lib).\n\nWraps multiple docker-compose commands to provide a one-stop tool.\n\nProvides the `http` CLI utility tool as a more specific and less cryptic alternative to `curl`.\n',
    'author': 'BrewPi',
    'author_email': 'development@brewpi.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.5',
}


setup(**setup_kwargs)
